--
-- EngineOilAddon
--
-- @author  
-- @author Cyprus Simulation Gamers
-- @date 06 Apr 2024
--
local modName = g_currentModName

EngineOilAddon = {
    modDirectory = g_currentModDirectory or "unknown",
    SPEC_TABLE_NAME = "spec_" .. modName .. ".engineOilAddon"
}

function EngineOilAddon.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(FillUnit, specializations) and
               SpecializationUtil.hasSpecialization(Motorized, specializations) and
               SpecializationUtil.hasSpecialization(Enterable, specializations) and
               SpecializationUtil.hasSpecialization(Drivable, specializations)
end

function EngineOilAddon.registerEventListeners(vehicleType)
    SpecializationUtil.registerEventListener(vehicleType, "onLoad",
                                             EngineOilAddon)
    SpecializationUtil.registerEventListener(vehicleType, "onLoadFinished",
                                             EngineOilAddon)
    SpecializationUtil.registerEventListener(vehicleType, "onUpdate",
                                             EngineOilAddon)
    SpecializationUtil.registerEventListener(vehicleType, "onDraw",
                                             EngineOilAddon)
end

function EngineOilAddon.registerFunctions(vehicleType)
    SpecializationUtil.registerFunction(vehicleType, "getEngineOilHudValues",
                                        EngineOilAddon.getEngineOilHudValues)
    SpecializationUtil.registerFunction(vehicleType,
                                        "getIsEngineOilInitialized",
                                        EngineOilAddon.getIsEngineOilInitialized)
    SpecializationUtil.registerFunction(vehicleType,
                                        "getCurrentFillTriggerFillType",
                                        EngineOilAddon.getCurrentFillTriggerFillType)
    SpecializationUtil.registerFunction(vehicleType, "getOilFillUnitIndex",
                                        EngineOilAddon.getOilFillUnitIndex)
    SpecializationUtil.registerFunction(vehicleType, "findOilCapacity",
                                        EngineOilAddon.findOilCapacity)
    SpecializationUtil.registerFunction(vehicleType,
                                        "loadEngineOilFillUnitFromXML",
                                        EngineOilAddon.loadEngineOilFillUnitFromXML)
end

function EngineOilAddon.initSpecialization()
    g_configurationManager:addConfigurationType("engineOilAddon",
                                                g_i18n:getText("name_engineOil"),
                                                nil, nil, nil, nil,
                                                ConfigurationUtil.SELECTOR_MULTIOPTION)
    StoreItemUtil.getConfigurationsFromXML =
        Utils.overwrittenFunction(StoreItemUtil.getConfigurationsFromXML,
                                  EngineOilUtils.getConfigurationsFromXML)

    local schema = Vehicle.xmlSchema
    local fillUnitPath = FillUnit.FILL_UNIT_XML_KEY
    schema:setXMLSpecializationType("EngineOilAddon")
    schema:register(XMLValueType.FLOAT,
                    fillUnitPath .. "#oilChangeIntervalHours",
                    "Oil change interval in hours")
    schema:setXMLSpecializationType()
end

function EngineOilAddon:onLoad(savegame)
    local spec = self[EngineOilAddon.SPEC_TABLE_NAME]
    local specFillunit = self.spec_fillUnit

    -- initialize
    spec.isInitializedOil = true
    spec.oilChangeAllowed = false
    spec.needFinishOilChange = false
    spec.showOilWarning = false
    spec.showOilChangeNotAllowed = false
    spec.stopVehicle = false
    spec.oilFillUnitIndex = nil
    spec.vehicleStopped = false
    spec.nextOilChange = 0

    local oilFillUnitsCount = 0
    for i = 1, #specFillunit.fillUnits do
        if self:getFillUnitAllowsFillType(i, FillType.ENGINEOIL) then
            oilFillUnitsCount = oilFillUnitsCount + 1
        end
    end

    if self.configurations.engineOilAddon == 1 then
        if oilFillUnitsCount == 0 then
            local xmlFillUnit = XMLFile.load("vehicleXml", Utils.getFilename(
                                                 "xmlFilename/mod.xml",
                                                 EngineOilAddon.modDirectory),
                                             Vehicle.xmlSchema)
            local newFillUnit = {}
            if self:loadEngineOilFillUnitFromXML(xmlFillUnit, string.format(
                                                     "vehicle.fillUnit.fillUnitConfigurations.fillUnitConfiguration(0).fillUnits.fillUnit(0)"),
                                                 newFillUnit,
                                                 #specFillunit.fillUnits + 1) then
                newFillUnit.capacity = self:findOilCapacity()
                newFillUnit.startFillLevel = self:findOilCapacity()
                table.insert(specFillunit.fillUnits, newFillUnit)
                spec.oilFillUnitIndex = #specFillunit.fillUnits
            end
        elseif oilFillUnitsCount > 0 then
            spec.oilFillUnitIndex = self:getOilFillUnitIndex()
        else
            spec.isInitializedOil = false
        end
    else
        spec.isInitializedOil = false
    end

    if spec.oilFillUnitIndex ~= nil and spec.isInitializedOil then
        spec.oilChangeHoursInterval = Utils.getNoNil(
                                          specFillunit.fillUnits[spec.oilFillUnitIndex]
                                              .oilChangeIntervalHours, 10)
        spec.oilChangeMinutesInterval = spec.oilChangeHoursInterval * 60
        spec.oilCapacity = specFillunit.fillUnits[spec.oilFillUnitIndex]
                               .capacity
        spec.currentOilFillLevel = self:getFillUnitFillLevel(
                                       spec.oilFillUnitIndex)
    end
end

function EngineOilAddon:onLoadFinished(savegame)
    local spec = self[EngineOilAddon.SPEC_TABLE_NAME]

    if spec.isInitializedOil then
        spec.lastOilLevelChange = math.floor(self.operatingTime / 60000)

        local currentOperMins = self.operatingTime / 60000
        spec.operatingTimeRemaining = (spec.oilChangeMinutesInterval *
                                          spec.currentOilFillLevel) /
                                          spec.oilCapacity
        spec.nextOilChange = spec.operatingTimeRemaining + currentOperMins
    end
end

function EngineOilAddon:onUpdate(dt, isActiveForInput,
                                 isActiveForInputIgnoreSelection, isSelected)
    local spec = self[EngineOilAddon.SPEC_TABLE_NAME]
    local specFillunit = self.spec_fillUnit
    local specMotor = self.spec_motorized

    local currentOperatingMinutes = math.floor(self.operatingTime / 60000)

    if spec.isInitializedOil then
        local oilFillLevel = self:getFillUnitFillLevel(spec.oilFillUnitIndex)

        -- Update next oil change time
        local currentOperMins = self.operatingTime / 60000
        spec.operatingTimeRemaining = (spec.oilChangeMinutesInterval *
                                          oilFillLevel) / spec.oilCapacity
        spec.nextOilChange = spec.operatingTimeRemaining + currentOperMins
        if spec.operatingTimeRemaining <= 0 then
            spec.oilOperatedMins = 0
        else
            spec.oilOperatedMins = spec.oilChangeMinutesInterval -
                                       spec.operatingTimeRemaining
        end
        -- Consume Oil
        if oilFillLevel > 0 and self:getIsOperating() then
            if currentOperatingMinutes >= spec.lastOilLevelChange + 1 then
                local fillDelta = (spec.oilCapacity /
                                      spec.oilChangeMinutesInterval) *
                                      (currentOperatingMinutes -
                                          spec.lastOilLevelChange)
                self:addFillUnitFillLevel(self:getOwnerFarmId(),
                                          spec.oilFillUnitIndex, -fillDelta,
                                          self:getFillUnitFillType(
                                              spec.oilFillUnitIndex),
                                          ToolType.UNDEFINED, unloadInfo)
                spec.lastOilLevelChange = currentOperatingMinutes
            end
        end

        spec.showOilWarning = false
        spec.showOilChangeNotAllowed = false

        if self:getFillUnitFillLevelPercentage(spec.oilFillUnitIndex) <= 0.1 then
            -- Allow change when below 10%
            spec.oilChangeAllowed = true;
        end

        -- Full
        if self:getFillUnitFillLevelPercentage(spec.oilFillUnitIndex) == 1 then
            spec.oilChangeAllowed = false
            spec.needFinishOilChange = false
            spec.stopVehicle = false
        end

        if specFillunit.fillTrigger.isFilling and
            self:getCurrentFillTriggerFillType() ==
            g_fillTypeManager:getFillTypeIndexByName("ENGINEOIL") then
            if spec.oilChangeAllowed then
                spec.needFinishOilChange = true
                specFillunit.fillTrigger.isFilling =
                    specFillunit.fillTrigger.isFilling
            else
                self:setFillUnitIsFilling(false)
                if self.isClient then
                    spec.showOilChangeNotAllowed = true
                end
            end
        end

        if oilFillLevel <= 0 or spec.needFinishOilChange then
            if self.isClient then spec.showOilWarning = true end
            spec.stopVehicle = true
        else
            spec.stopVehicle = false
        end

        if spec.stopVehicle then
            if self.isHired then self:stopAIVehicle(4) end
            self:stopMotor()
            self:stopVehicle()
            if not spec.vehicleStopped then
                self:deactivate()
                spec.vehicleStopped = true
            else
            end
        else
            self:activate()
        end
    end
end

function EngineOilAddon:onDraw(isActiveForInput,
                               isActiveForInputIgnoreSelection, isSelected)
    local spec = self[EngineOilAddon.SPEC_TABLE_NAME]

    if isActiveForInputIgnoreSelection and spec.isInitializedOil then
        if self:getFillUnitFillLevel(spec.oilFillUnitIndex) <= 0 then
            g_currentMission:addExtraPrintText(g_i18n:getText(
                                                   "info_firstFillEngineOil",
                                                   self.customEnvironment))
        elseif spec.needFinishOilChange then
            g_currentMission:addExtraPrintText(g_i18n:getText(
                                                   "info_needFinishFillOil",
                                                   self.customEnvironment))
        end
        if spec.showOilWarning then
            if spec.needFinishOilChange then
                g_currentMission:showBlinkingWarning(g_i18n:getText(
                                                         "info_needFinishFillOil",
                                                         self.customEnvironment),
                                                     4000)
            else
                g_currentMission:showBlinkingWarning(g_i18n:getText(
                                                         "info_firstFillEngineOil",
                                                         self.customEnvironment),
                                                     2000)
            end
        end
        if spec.showOilChangeNotAllowed then
            g_currentMission:showBlinkingWarning(g_i18n:getText(
                                                     "info_fillEngineOilNotAllowed",
                                                     self.customEnvironment),
                                                 2000)
        end
    end
end

function EngineOilAddon:getEngineOilHudValues()
    local spec = self[EngineOilAddon.SPEC_TABLE_NAME]
    local nextChange = EngineOilUtils.getTimeFormatted(spec.nextOilChange)
    local operMins = EngineOilUtils.getTimeFormatted(spec.oilOperatedMins)

    return {nextChange, operMins}
end

function EngineOilAddon:getIsEngineOilInitialized()
    local spec = self[EngineOilAddon.SPEC_TABLE_NAME]
    return spec.isInitializedOil
end

function EngineOilAddon:getCurrentFillTriggerFillType()
    local spec = self.spec_fillUnit
    if spec.fillTrigger.currentTrigger ~= nil then
        return spec.fillTrigger.currentTrigger:getCurrentFillType()
    else
        return 0
    end
end

function EngineOilAddon:findOilCapacity()
    local oilCapacity = 5
    local maxSpeed = self.spec_motorized.motor:getMaximumForwardSpeed() * 3.7
    local weight = self:getDefaultMass() * 1000

    if maxSpeed <= 15 and weight >= 12000 then
        oilCapacity = 55
    elseif maxSpeed < 100 and weight >= 12000 then
        oilCapacity = 45
    elseif weight > 1000 and maxSpeed < 60 then
        oilCapacity = 25
    elseif maxSpeed > 100 then
        oilCapacity = 9
    end

    return oilCapacity
end

function EngineOilAddon:getOilFillUnitIndex()
    local spec = self.spec_fillUnit
    local oilFillType = g_fillTypeManager:getFillTypeIndexByName("ENGINEOIL")

    for fillUnitIndex, _ in ipairs(spec.fillUnits) do
        if self:getFillUnitAllowsFillType(fillUnitIndex, oilFillType) then
            return fillUnitIndex
        end
    end
    return nil
end

function EngineOilAddon:loadEngineOilFillUnitFromXML(xmlFile, key, entry, index)
    local spec = self.spec_fillUnit
    entry.fillUnitIndex = index
    entry.capacity = xmlFile:getValue(key .. "#capacity", math.huge)
    entry.defaultCapacity = entry.capacity
    entry.updateMass = xmlFile:getValue(key .. "#updateMass", true)
    entry.canBeUnloaded = xmlFile:getValue(key .. "#canBeUnloaded", true)
    entry.allowFoldingThreshold = xmlFile:getValue(key ..
                                                       "#allowFoldingThreshold",
                                                   entry.capacity)
    entry.needsSaving = true
    entry.fillLevel = 0
    entry.fillLevelSent = 0
    entry.fillType = FillType.UNKNOWN
    entry.fillTypeSent = FillType.UNKNOWN
    entry.fillTypeToDisplay = FillType.UNKNOWN
    entry.fillLevelToDisplay = nil
    entry.lastValidFillType = FillType.UNKNOWN
    entry.lastValidFillTypeSent = FillType.UNKNOWN
    entry.oilChangeIntervalHours = xmlFile:getValue(key ..
                                                        "#oilChangeIntervalHours",
                                                    10)

    if xmlFile:hasProperty(key .. ".exactFillRootNode") then
        XMLUtil.checkDeprecatedXMLElements(xmlFile,
                                           key .. ".exactFillRootNode#index",
                                           key .. ".exactFillRootNode#node")
        entry.exactFillRootNode = xmlFile:getValue(key ..
                                                       ".exactFillRootNode#node",
                                                   nil, self.components,
                                                   self.i3dMappings)
        if entry.exactFillRootNode ~= nil then
            if not CollisionFlag.getHasFlagSet(entry.exactFillRootNode,
                                               CollisionFlag.FILLABLE) then
                Logging.xmlWarning(self.xmlFile,
                                   "Missing collision mask bit '%d'. Please add this bit to exact fill root node '%s' in '%s'",
                                   CollisionFlag.getBit(CollisionFlag.FILLABLE),
                                   getName(entry.exactFillRootNode), key)
                return false
            end
            spec.exactFillRootNodeToFillUnit[entry.exactFillRootNode] = entry
            spec.exactFillRootNodeToExtraDistance[entry.exactFillRootNode] =
                xmlFile:getValue(key .. ".exactFillRootNode#extraEffectDistance",
                                 0)
            spec.hasExactFillRootNodes = true
        else
            Logging.xmlWarning(self.xmlFile,
                               "ExactFillRootNode not found for fillUnit '%s'!",
                               key)
        end
    end
    XMLUtil.checkDeprecatedXMLElements(xmlFile,
                                       key .. ".autoAimTargetNode#index",
                                       key .. ".autoAimTargetNode#node")
    entry.autoAimTarget = {
        node = xmlFile:getValue(key .. ".autoAimTargetNode#node", nil,
                                self.components, self.i3dMappings)
    }
    if entry.autoAimTarget.node ~= nil then
        entry.autoAimTarget.baseTrans = {
            getTranslation(entry.autoAimTarget.node)
        }
        entry.autoAimTarget.startZ = xmlFile:getValue(key ..
                                                          ".autoAimTargetNode#startZ")
        entry.autoAimTarget.endZ = xmlFile:getValue(key ..
                                                        ".autoAimTargetNode#endZ")
        entry.autoAimTarget.startPercentage =
            xmlFile:getValue(key .. ".autoAimTargetNode#startPercentage", 25) /
                100
        entry.autoAimTarget.invert = xmlFile:getValue(key ..
                                                          ".autoAimTargetNode#invert",
                                                      false)
        if entry.autoAimTarget.startZ ~= nil and entry.autoAimTarget.endZ ~= nil then
            local startZ = entry.autoAimTarget.startZ
            if entry.autoAimTarget.invert then
                startZ = entry.autoAimTarget.endZ
            end
            setTranslation(entry.autoAimTarget.node,
                           entry.autoAimTarget.baseTrans[1],
                           entry.autoAimTarget.baseTrans[2], startZ)
        end
    end
    entry.supportedFillTypes = {}
    local fillTypes = nil
    local fillTypeCategories = xmlFile:getValue(key .. "#fillTypeCategories")
    local fillTypeNames = xmlFile:getValue(key .. "#fillTypes")
    if fillTypeCategories ~= nil and fillTypeNames == nil then
        fillTypes = g_fillTypeManager:getFillTypesByCategoryNames(
                        fillTypeCategories,
                        "Warning: '" .. self.configFileName ..
                            "' has invalid fillTypeCategory '%s'.")
    elseif fillTypeCategories == nil and fillTypeNames ~= nil then
        fillTypes = g_fillTypeManager:getFillTypesByNames(fillTypeNames,
                                                          "Warning: '" ..
                                                              self.configFileName ..
                                                              "' has invalid fillType '%s'.")
    else
        Logging.xmlWarning(self.xmlFile,
                           "Missing 'fillTypeCategories' or 'fillTypes' for fillUnit '%s'",
                           key)
        return false
    end
    if fillTypes ~= nil then
        for _, fillType in pairs(fillTypes) do
            entry.supportedFillTypes[fillType] = true
        end
    end
    entry.supportedToolTypes = {}
    for i = 1, g_toolTypeManager:getNumberOfToolTypes() do
        entry.supportedToolTypes[i] = true
    end
    local startFillLevel = xmlFile:getValue(key .. "#startFillLevel")
    local startFillTypeStr = xmlFile:getValue(key .. "#startFillType")
    if startFillTypeStr ~= nil then
        local startFillTypeIndex = g_fillTypeManager:getFillTypeIndexByName(
                                       startFillTypeStr)
        if startFillTypeIndex ~= nil then
            entry.startFillLevel = startFillLevel
            entry.startFillTypeIndex = startFillTypeIndex
        end
    end
    entry.fillRootNode = xmlFile:getValue(key .. ".fillRootNode#node", nil,
                                          self.components, self.i3dMappings)
    if entry.fillRootNode == nil then
        entry.fillRootNode = self.components[1].node
    end
    entry.fillMassNode = xmlFile:getValue(key .. ".fillMassNode#node", nil,
                                          self.components, self.i3dMappings)
    local updateFillLevelMass = xmlFile:getValue(key .. "#updateFillLevelMass",
                                                 true)
    if entry.fillMassNode == nil and updateFillLevelMass then
        entry.fillMassNode = self.components[1].node
    end
    entry.ignoreFillLimit = xmlFile:getValue(key .. "#ignoreFillLimit", false)
    entry.synchronizeFillLevel = xmlFile:getValue(
                                     key .. "#synchronizeFillLevel", true)
    entry.synchronizeFullFillLevel = xmlFile:getValue(key ..
                                                          "#synchronizeFullFillLevel",
                                                      false)
    local defaultBits = 16
    for startCapacity, bits in pairs(FillUnit.CAPACITY_TO_NETWORK_BITS) do
        if startCapacity <= entry.capacity then defaultBits = bits end
    end
    entry.synchronizationNumBits = xmlFile:getValue(key ..
                                                        "#synchronizationNumBits",
                                                    defaultBits)
    entry.showOnHud = xmlFile:getValue(key .. "#showOnHud", true)
    entry.showOnInfoHud = xmlFile:getValue(key .. "#showOnInfoHud", true)
    entry.uiPrecision = xmlFile:getValue(key .. "#uiPrecision", 0)
    local unitText = xmlFile:getValue(key .. "#unitTextOverride")
    if unitText ~= nil then entry.unitText = g_i18n:convertText(unitText) end
    entry.parentUnitOnHud = nil
    entry.childUnitOnHud = nil
    entry.blocksAutomatedTrainTravel = xmlFile:getValue(key ..
                                                            "#blocksAutomatedTrainTravel",
                                                        false)
    entry.fillAnimation = xmlFile:getValue(key .. "#fillAnimation")
    entry.fillAnimationLoadTime = xmlFile:getValue(key ..
                                                       "#fillAnimationLoadTime")
    entry.fillAnimationEmptyTime = xmlFile:getValue(key ..
                                                        "#fillAnimationEmptyTime")
    entry.fillLevelAnimationName = xmlFile:getValue(key ..
                                                        ".fillLevelAnimation#name")
    entry.fillLevelAnimationResetOnEmpty =
        xmlFile:getValue(key .. ".fillLevelAnimation#resetOnEmpty", true)
    if self.isClient then
        entry.alarmTriggers = {}
        local i = 0
        while true do
            local nodeKey = key ..
                                string.format(".alarmTriggers.alarmTrigger(%d)",
                                              i)
            if not xmlFile:hasProperty(nodeKey) then break end
            local alarmTrigger = {}
            if self:loadAlarmTrigger(xmlFile, nodeKey, alarmTrigger, entry) then
                table.insert(entry.alarmTriggers, alarmTrigger)
            end
            i = i + 1
        end
        entry.measurementNodes = {}
        i = 0
        while true do
            local nodeKey = key ..
                                string.format(
                                    ".measurementNodes.measurementNode(%d)", i)
            if not xmlFile:hasProperty(nodeKey) then break end
            local measurementNode = {}
            if self:loadMeasurementNode(xmlFile, nodeKey, measurementNode) then
                table.insert(entry.measurementNodes, measurementNode)
            end
            i = i + 1
        end
        entry.fillPlane = {}
        entry.lastFillPlaneType = nil
        if not self:loadFillPlane(xmlFile, key .. ".fillPlane", entry.fillPlane,
                                  entry) then entry.fillPlane = nil end
        entry.fillTypeMaterials = self:loadFillTypeMaterials(xmlFile, key)
        entry.fillEffects = g_effectManager:loadEffect(xmlFile,
                                                       key .. ".fillEffect",
                                                       self.components, self,
                                                       self.i3dMappings)
        entry.animationNodes = g_animationManager:loadAnimations(xmlFile,
                                                                 key ..
                                                                     ".animationNodes",
                                                                 self.components,
                                                                 self,
                                                                 self.i3dMappings)
        XMLUtil.checkDeprecatedXMLElements(xmlFile, key .. ".fillLevelHud",
                                           key .. ".dashboard")
        if self.loadDashboardsFromXML ~= nil then
            self:loadDashboardsFromXML(xmlFile, key, {
                maxFunc = "capacity",
                valueFunc = "fillLevel",
                minFunc = 0,
                valueTypeToLoad = "fillLevel",
                valueObject = entry
            })
            self:loadDashboardsFromXML(xmlFile, key, {
                maxFunc = 100,
                minFunc = 0,
                valueTypeToLoad = "fillLevelPct",
                valueObject = entry,
                valueFunc = function(fillUnit)
                    return MathUtil.clamp(
                               fillUnit.fillLevel / fillUnit.capacity, 0, 1) *
                               100
                end
            })
            self:loadDashboardsFromXML(xmlFile, key, {
                maxFunc = "capacity",
                valueFunc = "fillLevel",
                minFunc = 0,
                valueTypeToLoad = "fillLevelWarning",
                valueObject = entry,
                additionalAttributesFunc = Dashboard.warningAttributes,
                stateFunc = Dashboard.warningState
            })
        end
    end
    return true
end

function EngineOilAddon:updateDebugValues(values)
    local spec = self[EngineOilAddon.SPEC_TABLE_NAME]

    table.insert(values,
                 {name = "isInitializedOil", value = spec.isInitializedOil})
    if spec.isInitializedOil then
        table.insert(values, {
            name = "Oil FillUnit Index",
            value = string.format("%.4f", spec.oilFillUnitIndex)
        })
        table.insert(values, {
            name = "Trigger FillType Index",
            value = string.format("%.2f", self:getCurrentFillTriggerFillType())
        })
        table.insert(values, {
            name = "Engine Oil FillType Index",
            value = string.format("%.2f",
                                  g_fillTypeManager:getFillTypeIndexByName(
                                      "ENGINEOIL"))
        })
        table.insert(values, {
            name = "oilChangeHoursInterval",
            value = string.format("%.4f", spec.oilChangeHoursInterval)
        })
        table.insert(values, {
            name = "currentOperatingMinutes",
            value = string.format("%.3f", math.floor(self.operatingTime / 60000))
        })
        table.insert(values, {
            name = "nextOilChange",
            value = string.format("%.3f", spec.nextOilChange)
        })
        table.insert(values, {
            name = "oilFillLevelPercentage",
            value = string.format("%.3f", self:getFillUnitFillLevelPercentage(
                                      spec.oilFillUnitIndex))
        })
        table.insert(values, {
            name = "lastOilLevelChange",
            value = string.format("%.3f", spec.lastOilLevelChange)
        })
        table.insert(values,
                     {name = "oilChangeAllowed", value = spec.oilChangeAllowed})
        table.insert(values, {
            name = "needFinishOilChange",
            value = spec.needFinishOilChange
        })
        table.insert(values, {name = "stopVehicle", value = spec.stopVehicle})
    end
end
