--
-- EngineOilDisplay
-- Engine Oil hud display
-- 
-- @author Cyprus Simulation Gamers
-- @date 06 Apr 2024
-- 
local modDirectory = g_currentModDirectory
local modName = g_currentModName
EngineOilDisplay = {
    ---Element sizes
    SIZE = {SELF = {256, 64}, TEXT = 15, ICON = {40, 40}},

    ---Element positions
    POSITION = {
--        OFFSET = {-100, 280},
        OFFSET = {-100, 330},
        TEXT_OFFSET = {243, 17},
        ICON_OFFSET = {13, 12}
    },

    ---Element colors
    COLOR = {
        TEXT = {1, 1, 1, 1},
        TEXT_BACKGROUND = {0.15, 0.15, 0.15, 1},
        ICON = {0.0003, 0.5647, 0.9822, 1}
    }
}

local EngineOilDisplay_mt = Class(EngineOilDisplay, HUDDisplayElement)

function EngineOilDisplay.new()
    local backgroundOverlay = EngineOilDisplay.createBackground()
    local self = EngineOilDisplay:superClass().new(backgroundOverlay, nil,
                                                   EngineOilDisplay_mt)

    self.vehicle = nil
    self:applyValues(1)

    return self
end

---Sets the current vehicle
-- @param table vehicle Vehicle reference or nil (not controlling a vehicle)
function EngineOilDisplay:setVehicle(vehicle)
    -- check if the passed vehicle has engine oil addon
    if vehicle ~= nil and vehicle.getEngineOilHudValues == nil then
        vehicle = nil
    end

    self.vehicle = vehicle
end

---Draws the engine oil display
function EngineOilDisplay:draw()
    if self.vehicle == nil then return end

    if not self.vehicle:getIsEngineOilInitialized() then return end

    -- Class the draw function of the super class to render the background
    EngineOilDisplay:superClass().draw(self)

    -- Get the current driven distance of the vehicle
    local engineOilTexts = self.vehicle:getEngineOilHudValues()

    local nextOilChange = engineOilTexts[1] or ""
    local oilOperatedMins = engineOilTexts[2] or ""

    local text = g_i18n:getText("info_nextOilChange", self.customEnvironment) ..
                     ": " .. nextOilChange
    local text2 = g_i18n:getText("info_currentOil", self.customEnvironment) ..
                      ": " .. oilOperatedMins

    local textBG =
        string.rep("W", math.max(string.len(text), string.len(text2)))

    -- Store the color in a local for faster access
    local textColor = EngineOilDisplay.COLOR.TEXT
    local textColorBG = EngineOilDisplay.COLOR.TEXT_BACKGROUND
    local textSize = self.textSize

    -- Get the position of the display
    local posX, posY = self:getPosition()
    local iconPosX = posX + self.iconOffsetX
    local iconPosY = posY + self.iconOffsetY

    posX = posX + self.textOffsetX
    posY = posY + self.textOffsetY

    -- Disable bold textrendering
    setTextBold(false)
    -- Set textrendering to right alignment
    setTextAlignment(RenderText.ALIGN_RIGHT)

    -- Set the color of the textrendering for the text
    setTextColor(textColor[1], textColor[2], textColor[3], textColor[4])
    -- Render the text
    renderText(posX, posY, textSize, text)

    -- Move to bottom
    posY = posY + getTextHeight(textSize, text)
    renderText(posX, posY, textSize, text2)

    -- As text rendering functions are using a global scope we need to reset all our changes
    -- Reset text alignment to left
    setTextAlignment(RenderText.ALIGN_LEFT)
    -- Reset text color to white
    setTextColor(1, 1, 1, 1)

    -- Render Icon
    local oilIcon = EngineOilDisplay.createIcon()

    renderOverlay(oilIcon, iconPosX, iconPosY, self.iconWidth, self.iconHeight)
end

-- Create Icon Element
function EngineOilDisplay.createIcon()
    local r, g, b, a = unpack(EngineOilDisplay.COLOR.ICON)
    local oilIconElement = createImageOverlay(Utils.getFilename(
                                                  "image/gui_Icon_EngineOil.dds",
                                                  modDirectory))
    setOverlayColor(oilIconElement, r, g, b, a)
    return oilIconElement
end
--[[ ================= Scaling Functions  ================= ]]

---Set this element's UI scale factor.
-- @param float uiScale UI scale factor
-- @includeCode
function EngineOilDisplay:setScale(uiScale)
    EngineOilDisplay:superClass().setScale(self, uiScale, uiScale)

    -- Reposition the mileage display if ui scale changed
    local posX, posY = EngineOilDisplay.getBackgroundPosition(uiScale)
    self:setPosition(posX, posY)

    -- Recalculate the offset values
    self:applyValues(uiScale)
end

---Calculate some text values with a given ui scale
-- @param float uiScale UI scale factor
-- @includeCode
function EngineOilDisplay:applyValues(uiScale)
    -- Convert our pixel values stored in MileageDisplay.POSITION.TEXT_OFFSET to normalized screen values
    -- getNormalizedScreenValues also take care abour aspect ratios. It assumes that the hud/ui is designed for Full-HD (1920x1080)
    local textOffsetX, textOffsetY = getNormalizedScreenValues(unpack(
                                                                   EngineOilDisplay.POSITION
                                                                       .TEXT_OFFSET))
    local _, textSize = getNormalizedScreenValues(0, EngineOilDisplay.SIZE.TEXT)

    local iconWidth, iconHeight = getNormalizedScreenValues(unpack(
                                                                EngineOilDisplay.SIZE
                                                                    .ICON))
    local iconOffsetX, iconOffsetY = getNormalizedScreenValues(unpack(
                                                                   EngineOilDisplay.POSITION
                                                                       .ICON_OFFSET))

    self.textOffsetX = textOffsetX * uiScale
    self.textOffsetY = textOffsetY * uiScale
    self.textSize = textSize * uiScale

    self.iconOffsetX = iconOffsetX * uiScale
    self.iconOffsetY = iconOffsetY * uiScale
    self.iconWidth = iconWidth * uiScale
    self.iconHeight = iconHeight * uiScale
end

---Get the scaled background position.
-- @param float uiScale UI scale factor
function EngineOilDisplay.getBackgroundPosition(uiScale)
    local width, _ = getNormalizedScreenValues(
                         unpack(EngineOilDisplay.SIZE.SELF))
    local offsetX, offsetY = getNormalizedScreenValues(unpack(
                                                           EngineOilDisplay.POSITION
                                                               .OFFSET))
    local posX = 1 - width * uiScale + offsetX * uiScale
    local posY = offsetY * uiScale

    return posX, posY
end

---Create the background overlay.
function EngineOilDisplay.createBackground()
    local posX, posY = EngineOilDisplay.getBackgroundPosition(1)
    local width, height = getNormalizedScreenValues(unpack(
                                                        EngineOilDisplay.SIZE
                                                            .SELF))

    -- We need to convert the given filepath and add the mod directory to get the final absolute path
    local filename = Utils.getFilename("image/gui_EngineOil_background.dds",
                                       modDirectory)

    -- Creating a new overlay
    local overlay = Overlay.new(filename, posX, posY, width, height)
    return overlay
end
